@@ -4,19 +4,52 @@
 setMethod("read.dataset",
           c("BNDataset", "character", "character"),
           function(object, data.file, header.file, data.with.header = FALSE,
-                   na.string.symbol = '?', sep.symbol = '', starts.from = 1)
+                   na.string.symbol = '?', sep.symbol = '', starts.from = 1,
+                   num.time.steps = 1)
           {
             header.file(object)  <- header.file
             data.file(object)    <- data.file
-            
-            ls                   <- readLines(header.file)
-            variables(object)    <- gsub('"', '', c(unlist(strsplit(ls[1], split = " "))))
-            lns                  <- c(unlist(strsplit(ls[2], split = " ")))
-            node.sizes(object)   <- sapply(1:length(lns), FUN=function(x){ as.numeric(lns[x]) })
-            discreteness(object) <- c(unlist(strsplit(ls[3], split = " ")))
-            
+
             a <- read.delim(data.file, na.strings = na.string.symbol,
                             header = data.with.header, sep = sep.symbol) + (1 - starts.from)
+            a <- as.matrix(a)
+            
+            ls                   <- readLines(header.file)
+
+            vars <- gsub('"', '', c(unlist(strsplit(ls[1], split = " "))))
+            if (length(vars) == ncol(a)) {
+              variables(object) <- vars
+            } else if (num.time.steps > 1 && length(vars) * num.time.steps == ncol(a)) {
+              copyvars <- c()
+              for (t in 1:num.time.steps) {
+                for (w in vars) {
+                  copyvars <- c(copyvars, paste(w, as.character(t), sep='_t'))
+                }
+              }
+              variables(object) <- copyvars
+            } else {
+              stop("Incoherent number of variables in the dataset header.")
+            }
+            
+            lns                  <- c(unlist(strsplit(ls[2], split = " ")))
+            if (length(lns) == ncol(a)) {
+              node.sizes(object)   <- sapply(1:length(lns), FUN=function(x){ as.numeric(lns[x]) })
+            } else if (num.time.steps > 1 && length(lns) * num.time.steps == ncol(a)) {
+              node.sizes(object)   <- rep(sapply(1:length(lns), FUN=function(x){ as.numeric(lns[x]) }), num.time.steps)
+            } else {
+              stop("Incoherent number of variables in the dataset header.")
+            }
+            
+            disc <- c(unlist(strsplit(ls[3], split = " ")))
+            if (length(disc) == ncol(a)) {
+              discreteness(object) <- disc
+            } else if (num.time.steps > 1 && length(disc) * num.time.steps == ncol(a)) {
+              discreteness(object) <-rep(disc, num.time.steps)
+            } else {
+              stop("Incoherent number of variables in the dataset header.")
+            }
+            
+            
             raw.data(object)      <- as.matrix(a)
             num.variables(object) <- ncol(object@raw.data)
             num.items(object)     <- nrow(object@raw.data)
@@ -527,11 +560,12 @@ setMethod("write.dsc","BN",
             write(rows, file=file.name)
           })
 
-#' @rdname write.xgmml
-#' @aliases write.xgmml,BN
-#' @importFrom grDevices col2rgb rgb
-setMethod("write.xgmml","BN",
-          function(x, filename="./", write.wpdag=FALSE, node.col = rep('white',num.nodes(x)),
+
+# ' @importFrom grDevices col2rgb rgb
+#' @rdname write_xgmml
+#' @aliases write_xgmml,BN
+setMethod("write_xgmml","BN",
+          function(x, path="./network", write.wpdag=FALSE, node.col = rep('white',num.nodes(x)),
                    frac = 0.2, max.weight=max(wpdag(x)))
           {
             
@@ -541,14 +575,14 @@ setMethod("write.xgmml","BN",
             }
             
             weight.color.hex <- function(weight) {
-              weight <- 255 - weight
+              # weight <- 255 - weight
               return(rgb(weight, weight, weight, maxColorValue=255))
             }
             
-            file.name <- strcat(filename, ".xgmml")
+            file.name <- strcat(path, ".xgmml")
             rows      <- NULL
             rows[[1]] <- "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>"
-            rows[[2]] <- "<graph label=\"Network\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:cy=\"http://www.cytoscape.org\" xmlns=\"http://www.cs.rpi.edu/XGMML\"  directed=\"1\">"
+            rows[[2]] <- "<graph label=\"Network\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:cy=\"http://www.cytoscape.org\" xmlns=\"http://www.cs.rpi.edu/XGMML\"  directed=\"1\" Layout=\"Tree\">"
             rows[[3]] <- "<att name=\"documentVersion\" value=\"1.1\"/>"
             k         <- 4
             
@@ -570,7 +604,7 @@ setMethod("write.xgmml","BN",
             # pass through an igraph to get the coordinates
             # otherwise cytoscape will put every node in the same place...
             ig.obj <- igraph::graph_from_adjacency_matrix(g)
-            coords <- layout_nicely(ig.obj) * 3 * num.nodes # also, rescale
+            coords <- igraph::layout_nicely(ig.obj) * 3 * num.nodes # also, rescale
             # coords <- layout_with_graphopt(ig.obj) * 3 * num.nodes # also, rescale
             
             for (node in 1:num.nodes)
@@ -583,7 +617,8 @@ setMethod("write.xgmml","BN",
               rows[[k+1]] <- s
               k           <- k+2
               rows[[k]]   <- strcat("    <graphics type=\"ELLIPSE\" h=\"40.0\" w=\"40.0\" x=\"",coords[node,1],
-                                    "\" y=\"",coords[node,2],"\" fill=\"",get.color.code(node.col[node]),
+                                    "\" y=\"",coords[node,2],"\" ",
+                                    " fill=\"",get.color.code(node.col[node]),
                                     "\" width=\"1\" outline=\"#666666\" cy:nodeTransparency=\"1.0\" cy:nodeLabel=\"",
                                     variables(x)[node],"\" cy:borderLineType=\"solid\"/>")
               k           <- k+1
@@ -596,7 +631,7 @@ setMethod("write.xgmml","BN",
               for (j in (i+1):num.nodes) {
                 # undirected edge
                 if (g[i,j] > 0 && g[j,i] > 0) {
-                  rows[[k]] <- strcat("<edge id=\"",edge.counter,"\" label=\"\" source=\"",i,"\" target=\"",j,"\" cy:directed=\"0\">")
+                  rows[[k]] <- strcat("<edge id=\"",edge.counter,"\" label=\"",edge.counter,"\" source=\"",i,"\" target=\"",j,"\" cy:directed=\"0\">")
                   k         <- k + 1
                   rows[[k]] <- "  <att name=\"shared name\" value=\"\" type=\"string\" cy:type=\"String\"/>"
                   k         <- k + 1
@@ -610,7 +645,7 @@ setMethod("write.xgmml","BN",
                   k         <- k + 1
                   rows[[k]] <- strcat("  <att name=\"name\" value=\"",i,"-",j,"\" type=\"string\" cy:type=\"String\"/>")
                   k         <- k + 1
-                  rows[[k]] <- "<att name=\"selected\" value=\"0\" type=\"boolean\" cy:type=\"Boolean\"/>"
+                  rows[[k]] <- "<att name=\"selected\" value=\"1\" type=\"boolean\" cy:type=\"Boolean\"/>"
                   k         <- k + 1
                   rows[[k]] <- "  <att name=\"interaction\" value=\"interacts with\" type=\"string\" cy:type=\"String\"/>"
                   k         <- k + 1
@@ -626,7 +661,7 @@ setMethod("write.xgmml","BN",
                   edge.counter <- edge.counter + 1
                 } else if (g[i,j] > 0 || g[j,i] > 0) { # directed edge
                   if (g[i,j] > 0 && g[j,i] == 0) {
-                    rows[[k]] <- strcat("<edge id=\"",edge.counter,"\" label=\"\" source=\"",i,"\" target=\"",j,"\" cy:directed=\"1\">")
+                    rows[[k]] <- strcat("<edge id=\"",edge.counter,"\" label=\"",edge.counter,"\" source=\"",i,"\" target=\"",j,"\" cy:directed=\"1\">")
                     k         <- k + 1
                     rows[[k]] <- "  <att name=\"shared name\" value=\"\" type=\"string\" cy:type=\"String\"/>"
                     k         <- k + 1
@@ -640,7 +675,7 @@ setMethod("write.xgmml","BN",
                     k         <- k + 1
                     rows[[k]] <- strcat("  <att name=\"name\" value=\"",i,"-",j,"\" type=\"string\" cy:type=\"String\"/>")
                     k         <- k + 1
-                    rows[[k]] <- "<att name=\"selected\" value=\"0\" type=\"boolean\" cy:type=\"Boolean\"/>"
+                    rows[[k]] <- "  <att name=\"selected\" value=\"1\" type=\"boolean\" cy:type=\"Boolean\"/>"
                     k         <- k + 1
                     rows[[k]] <- "  <att name=\"interaction\" value=\"interacts with\" type=\"string\" cy:type=\"String\"/>"
                     k         <- k + 1
@@ -654,7 +689,7 @@ setMethod("write.xgmml","BN",
                     k         <- k + 1
                     edge.counter <- edge.counter + 1
                   } else if (g[j,i] > 0 && g[i,j] == 0) {
-                    rows[[k]] <- strcat("<edge id=\"",edge.counter,"\" label=\"\" source=\"",j,"\" target=\"",i,"\" cy:directed=\"1\">")
+                    rows[[k]] <- strcat("<edge id=\"",edge.counter,"\" label=\"",edge.counter,"\" source=\"",j,"\" target=\"",i,"\" cy:directed=\"1\">")
                     k         <- k + 1
                     rows[[k]] <- "  <att name=\"shared name\" value=\"\" type=\"string\" cy:type=\"String\"/>"
                     k         <- k + 1
@@ -668,7 +703,7 @@ setMethod("write.xgmml","BN",
                     k         <- k + 1
                     rows[[k]] <- strcat("  <att name=\"name\" value=\"",j,"-",i,"\" type=\"string\" cy:type=\"String\"/>")
                     k         <- k + 1
-                    rows[[k]] <- "<att name=\"selected\" value=\"0\" type=\"boolean\" cy:type=\"Boolean\"/>"
+                    rows[[k]] <- "<att name=\"selected\" value=\"1\" type=\"boolean\" cy:type=\"Boolean\"/>"
                     k         <- k + 1
                     rows[[k]] <- "  <att name=\"interaction\" value=\"interacts with\" type=\"string\" cy:type=\"String\"/>"
                     k         <- k + 1
